import os

import hubble
from hubble.excepts import AuthenticationFailedError, AuthenticationRequiredError


def login(args):
    client = hubble.Client(max_retries=None)

    from rich.console import Console

    console = Console()

    if args.force:
        hubble.login(prompt='login')
        return

    try:
        resp = client.get_user_info()
        nickname = resp.json().get('data', {}).get('nickname', '')
        if nickname:
            console.print(
                f'You are already logged in as [b green]{nickname}[/b green].',
                '',
                'If you want to login to another account, please do either way:',
                '- run `jina auth logout` first',
                '- run `jina auth login -f`',
                sep=os.linesep,
            )
    except Exception as ex:
        if isinstance(ex, AuthenticationRequiredError) or isinstance(
            AuthenticationFailedError
        ):
            hubble.login(prompt='login')
        else:
            raise ex


def logout(args):
    hubble.logout()


def token(args):
    client = hubble.Client(max_retries=None)

    if args.operation == 'create':
        response = client.create_personal_access_token(
            name=args.name, expiration_days=args.expire
        )

        response.raise_for_status()
        token = response.json()['data']['token']

        import rich
        from rich.panel import Panel

        rich.print(
            Panel(
                f'token: [b]{token}[/b]',
                title=':party_popper: [green]Successfully created',
                subtitle=':point_up:️ [red]Please keep this token in a safe place!',
                width=50,
            )
        )

    if args.operation == 'delete':
        response = client.delete_personal_access_token(name=args.name)

        response.raise_for_status()

        import rich
        from rich.panel import Panel

        rich.print(
            Panel(
                f'[b]{args.name}[/b]',
                title=':party_popper: [green]Successfully deleted',
                width=50,
            )
        )

    if args.operation == 'list':
        response = client.list_personal_access_tokens()
        response.raise_for_status()
        tokens = response.json()['data']['personal_access_tokens']

        import rich
        from rich.table import Table

        table = Table(title="Your personal access tokens")
        table.add_column("ID", justify="center", style="cyan", no_wrap=True)
        table.add_column("Type", style="magenta")
        table.add_column("Name", justify="center", style="green")
        table.add_column("ExpireAt", justify="center", style="green")
        table.add_column("CreatedAt", justify="center", style="green")
        table.add_column("UpdatedAt", justify="center", style="green")

        for token in tokens:
            table.add_row(
                token['_id'],
                token['type'],
                token['name'],
                token['expireAt'],
                token['createdAt'],
                token['updatedAt'],
            )

        rich.print(table)



from argparse import ArgumentParser


def set_token_parser(parent_parser: 'ArgumentParser' = None):
    if not parent_parser:
        raise RuntimeError('Unreachable')

    from auth.parsers.helper import _chf

    token_parser = parent_parser.add_subparsers(
        dest='operation',
        required=True,
    )

    create_parser = token_parser.add_parser(
        'create',
        description='Create a Personal Access Token',
        formatter_class=_chf,
    )

    create_parser.add_argument(
        '-e',
        '--expire',
        type=int,
        default=7,
        help='Validity period (days)',
    )

    create_parser.add_argument(
        'name',
        type=str,
        help='Name of Personal Access Token',
    )

    delete_parser = token_parser.add_parser(
        'delete',
        description='Delete a Personal Access Token',
        formatter_class=_chf,
    )

    delete_parser.add_argument(
        'name',
        type=str,
        help='Name of Personal Access Token which you want to delete',
    )

    list_parser = token_parser.add_parser(  # noqa F841
        'list',
        description='List all Personal Access Tokens',
        formatter_class=_chf,
    )



import sys

from setuptools import find_packages, setup

if sys.version_info < (3, 7, 0):
    raise OSError(f'Requires Python >=3.7, but yours is {sys.version}')

try:
    with open('README.md', encoding='utf8') as fp:
        _long_description = fp.read()
except FileNotFoundError:
    _long_description = ''

# package requirements
try:
    with open('requirements.txt', 'r') as f:
        _install_requires = f.readlines()
except FileNotFoundError:
    _install_requires = []

setup(
    name='jina-auth',
    packages=find_packages(),
    include_package_data=True,
    description='A CLI for you to login in to Jina Ecosystem',
    author='Jina AI',
    author_email='hello@jina.ai',
    license='Apache 2.0',
    url='https://github.com/jina-ai/auth',
    download_url='https://github.com/jina-ai/auth/tags',
    long_description=_long_description,
    long_description_content_type='text/markdown',
    zip_safe=False,
    setup_requires=['setuptools>=18.0', 'wheel'],
    install_requires=_install_requires,
    extras_require={
        'test': [
            'pytest',
            'pytest-timeout',
            'pytest-mock',
            'pytest-cov',
            'pytest-repeat',
            'pytest-reraise',
            'mock',
            'pytest-custom_exit_code',
            'black==22.3.0',
            'jina',
            'flake8',
            'isort',
        ],
    },
    entry_points={
        'console_scripts': [
            'jina-auth=auth.__main__:main',
        ],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Unix Shell',
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Database :: Database Engines/Servers',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
        'Topic :: Scientific/Engineering :: Image Recognition',
        'Topic :: Multimedia :: Video',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    project_urls={
        'Documentation': 'https://github.com/jina-ai/auth/',
        'Source': 'https://github.com/jina-ai/auth/',
        'Tracker': 'https://github.com/jina-ai/auth/issues',
    },
    keywords='jina auth login logout user',
)
